package com.galaxis.manatee.dao;

import com.galaxis.capsule.util.GalaxisRepository;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunMemberCostDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectCostDO;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author kxuan
 */
@Repository
public interface ChuanyunProjectCostDao extends GalaxisRepository<ChuanyunProjectCostDO,String> {

    /**
     * 获取项目总工时成本
     * @return  项目总工时
     */
    @Query(value="SELECT chuanyun_member_cost.object_id," +
            "SUM(chuanyun_member_cost.cost) AS cost," +
            "SUM(chuanyun_member_cost.total_hour) AS total_hour," +
            "SUM(chuanyun_member_cost.person_days) AS person_days," +
            "chuanyun_member_cost.project_type," +
            "chuanyun_member_cost.status," +
            "chuanyun_member_cost.project_id, " +
            "chuanyun_member_cost.cost AS amount" +
            " FROM chuanyun_member_cost " +
            "GROUP BY chuanyun_member_cost.project_id,"+
            "chuanyun_member_cost.status",
            nativeQuery = true)
    List<ChuanyunProjectCostDO> getProjectCost();

    /**
     * 根据项目号获取工时数据
     * @param projectId 项目Id
     * @return  项目工时对象
     */
    ChuanyunProjectCostDO findByProjectId(String projectId);

    @Transactional
    void deleteAllByStatus(String status);
}
