package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.capsule.dto.PageDTO;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.manager.ClawFeign;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/4/27 5:14 下午
 */
@Service
@Slf4j
public class ChuanyunBasicDataScheduledTask {

    private final Integer pageSize=20;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunUserDao chuanyunUserDao;
    private final ChuanyunSupplierDao chuanyunSupplierDao;
    private final ChuanyunCompanyDao chuanyunCompanyDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunCostDao chuanyunCostDao;
    private final ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao;
    private final ChuanyunProjectTypeDao chuanyunProjectTypeDao;
    private final ChuanyunSpecialProcedureDao chuanyunSpecialProcedureDao;
    private final ChuanyunCostCollectionDao chuanyunCostCollectionDao;
    private final ChuanyunReimbursementItemsDao chuanyunReimbursementItemsDao;
    private final ClawFeign clawFeign;
    private final ChuanyunAskForLeaveDao chuanyunAskForLeaveDao;
    private final ChuanyunBudgetStatisticsDao chuanyunBudgetStatisticsDao;

    public ChuanyunBasicDataScheduledTask(ChuanYunManager chuanYunManager, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunUserDao chuanyunUserDao, ChuanyunSupplierDao chuanyunSupplierDao, ChuanyunCompanyDao chuanyunCompanyDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunCostDao chuanyunCostDao, ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao, ChuanyunProjectTypeDao chuanyunProjectTypeDao, ChuanyunSpecialProcedureDao chuanyunSpecialProcedureDao, ChuanyunCostCollectionDao chuanyunCostCollectionDao, ChuanyunReimbursementItemsDao chuanyunReimbursementItemsDao, ClawFeign clawFeign, ChuanyunAskForLeaveDao chuanyunAskForLeaveDao, ChuanyunBudgetStatisticsDao chuanyunBudgetStatisticsDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunUserDao = chuanyunUserDao;
        this.chuanyunSupplierDao = chuanyunSupplierDao;
        this.chuanyunCompanyDao = chuanyunCompanyDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunCostDao = chuanyunCostDao;
        this.chuanyunReimbursementSubjectDao = chuanyunReimbursementSubjectDao;
        this.chuanyunProjectTypeDao = chuanyunProjectTypeDao;
        this.chuanyunSpecialProcedureDao = chuanyunSpecialProcedureDao;
        this.chuanyunCostCollectionDao = chuanyunCostCollectionDao;
        this.chuanyunReimbursementItemsDao = chuanyunReimbursementItemsDao;
        this.clawFeign = clawFeign;
        this.chuanyunAskForLeaveDao = chuanyunAskForLeaveDao;
        this.chuanyunBudgetStatisticsDao = chuanyunBudgetStatisticsDao;
    }

    public void getReimbursementItemsManually() {
        this.getReimbursementItems();
    }

    public void getUserCompanyManually(){
        this.getUserCompany();
    }

    public void getUserListManually(){
        this.getUserList();
    }

    public void getCompanyManually(){
        this.getCompany();
    }

    public void getGroupProjectManually(){
        this.getGroupProject();
    }

    public void getCostCollectionManually() {this.getCostCollection();}

    public void getGroupProjectForSynchronizationManually() {this.getGroupProjectForSynchronization();}


    /**
     * 定时将氚云中的数据保存到manatee
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getDataFromChuanyun() {
        //项目预决算
        saveBudgetStatistics();
        //钉钉请假
        saveAskForLeave();
        //专项费用分摊
        getSpecialProcedure();
        //获取人员-归属公司信息
        getUserCompany();
        //人员报销特殊项目
        getReimbursementItems();
        //人工成本特殊项目
        getCostCollection();
        //获取集团项目数据
        getGroupProject();
        //获取用户数据
        getUserList();
        //支出档案类型
        getProjectType();
        //获取报销科目
        getReimbursementSubject();
        //获取成本中心
        getCost();
        //获取公司信息
        getCompany();
        //获取供应商
        getSupplier();
        //项目物料成本
        savePurCost();
        //项目收付款
        saveCashFlow();
        //新增供应商信息
        saveSupplier();
    }

    /**
     * 获取用户工号
     */
    @Scheduled(fixedDelay = 1800000L)
    private void getUserEmployeeNumber(){
        chuanyunUserDao.findAll().forEach(chuanyunUserDO -> {
            if(StringUtils.isEmpty(chuanyunUserDO.getEmployeeNumber())){
                ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),"");
            }else{
                ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),chuanyunUserDO.getEmployeeNumber());
            }
        });
        log.info("用户工号MAP更新完成");
    }

    /**
     * 从氚云中获取用户列表
     */
    private void getUserList() {
        var objectMapper=new ObjectMapper();
        var chuanyunFindAllDTO=chuanYunManager.findAll(ChuanyunUserDO.APP_CODE,ChuanyunUserDO.ACTION_NAME,ChuanyunUserDO.CONTROLLER);
        var totalCount=0;
        try {
            List<ChuanyunUserDO> userList=objectMapper.readValue(chuanyunFindAllDTO.getReturnData().getData(),new TypeReference<>(){});
            totalCount=chuanyunUserDao.saveAll(userList).size();
        } catch (Exception e) {
            e.printStackTrace();
        }
        log.info("用户信息保存保存"+totalCount+"工号映射更新完成");
    }

    /**
     * 从氚云获取集团项目信息
     */
    private void getGroupProject(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;

        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {
                    var tmpGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunGroupProjectDO.getObjectId());
                    tmpGroupProjectDO.ifPresent(projectDO -> {
                        chuanyunGroupProjectDO.setIsCreate(projectDO.getIsCreate());
                        chuanyunGroupProjectDao.delete(tmpGroupProjectDO.get());
                    });
                    //处理项目对象中的项目成员数据
                    projectMemberListCheck(chuanyunGroupProjectDO);
                    //处理项目对象中的项目经理、项目负责人数据
                    projectManagerCheck(chuanyunGroupProjectDO);
                    chuanyunGroupProjectDao.save(chuanyunGroupProjectDO);
                });
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("集团项目信息保存保存"+totalCount+"项目映射更新完成");
    }

    /**
     * 更新项目成员方法
     * @param chuanyunGroupProjectDO    项目对象
     */
    private void projectMemberListCheck(ChuanyunGroupProjectDO chuanyunGroupProjectDO){
        var objectMapper=new ObjectMapper();
        //第一步，清除项目负责人、项目经理数据。保证能够触发权限修改流程
        GroupProjectMemberListUpdateDTO groupProjectMemberListUpdateDTO=new GroupProjectMemberListUpdateDTO(chuanyunGroupProjectDO);
        if(groupProjectMemberListUpdateDTO.getProjectMemberList()!=null&&groupProjectMemberListUpdateDTO.getProjectMemberList().length>0){
            groupProjectMemberListUpdateDTO.add();
            try {
                chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),objectMapper.writeValueAsString(groupProjectMemberListUpdateDTO));
            } catch (Exception e) {
                e.printStackTrace();
                log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目成员失败"+e.getMessage());
            }
            groupProjectMemberListUpdateDTO.delete();
            //保证项目成员信息更新成功
            boolean updateFlag=false;
            while (!updateFlag){
                try {
                    var groupProjectMemberListUpdateString=objectMapper.writeValueAsString(groupProjectMemberListUpdateDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectMemberListUpdateString);
                    if(chuanyunSaveDTO.getSuccessful()){
                        updateFlag=true;
                    }else{
                        log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目成员失败"+groupProjectMemberListUpdateString);
                        log.warn(chuanyunGroupProjectDO+"");
                        Thread.sleep(5000L);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目成员失败"+e.getMessage());
                }
            }
        }
    }

    /**
     * 通过更新集团项目中的项目负责人、EBG项目经理、IBG项目经理
     * @param chuanyunGroupProjectDO    项目对象
     */
    private void projectManagerCheck(ChuanyunGroupProjectDO chuanyunGroupProjectDO){
        //将projectManger和projectOwner替换为主键
        if(!StringUtils.isEmpty(chuanyunGroupProjectDO.getProjectOwner())){
            chuanyunGroupProjectDO.setProjectOwner(chuanyunGroupProjectDO.getProjectOwnerObject().getObjectId());
        }
        if (!StringUtils.isEmpty(chuanyunGroupProjectDO.getEbgProjectManager())){
            chuanyunGroupProjectDO.setEbgProjectManager(chuanyunGroupProjectDO.getEbgProjectManagerObject().getObjectId());
        }
        if(!StringUtils.isEmpty(chuanyunGroupProjectDO.getIbgProjectManager())){
            chuanyunGroupProjectDO.setIbgProjectManager(chuanyunGroupProjectDO.getIbgProjectManagerObject().getObjectId());
        }
        if(StringUtils.isEmpty(chuanyunGroupProjectDO.getOutProjectName())) {
            chuanyunGroupProjectDO.setOutProjectName(chuanyunGroupProjectDO.getProjectName());
        }
        var objectMapper=new ObjectMapper();
        //第一步，清除项目负责人、项目经理数据。保证能够触发权限修改流程
        GroupProjectUpdateDTO groupProjectUpdateDTO=new GroupProjectUpdateDTO();
        try {
            chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),objectMapper.writeValueAsString(groupProjectUpdateDTO));
        } catch (Exception e) {
            e.printStackTrace();
            log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目经理失败"+e.getMessage());
        }
        groupProjectUpdateDTO=new GroupProjectUpdateDTO(chuanyunGroupProjectDO);
        //保证项目经理信息更新成功
        boolean updateFlag=false;
        while (!updateFlag){
            try {
                var groupProjectUpdateString=objectMapper.writeValueAsString(groupProjectUpdateDTO);
                ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectUpdateString);
                if(chuanyunSaveDTO.getSuccessful()){
                    updateFlag=true;
                    //更新成功之后重新获取集团项目组信息，防止id为空
                    var chuanyunFindResponse =  chuanYunManager.find(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId());
                    ChuanyunGroupProjectDO tmpChuanyunGroupProjectDO = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>(){});
                    chuanyunGroupProjectDO.setDetailList(tmpChuanyunGroupProjectDO.getDetailList());
                }else{
                    log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目经理失败"+groupProjectUpdateString);
                    log.warn(chuanyunGroupProjectDO+"");
                    Thread.sleep(5000L);
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目经理失败"+e.getMessage());
            }
        }
    }

    /**
     * 更新项目收付款
     */
    private void saveCashFlow() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取收付款
            PageDTO<ChuanyunProjectCashFlowDTO> pageInfo = clawFeign.findChuanyunProjectCashFlow(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunProjectCashFlowDTO> list = pageInfo.getContent();
            list.forEach(chuanyunProjectCashFlowDTO -> {
                try {
                    if(chuanyunProjectCashFlowDTO != null) {
                        CashFlowDTO cashFlowDTO = new CashFlowDTO(chuanyunProjectCashFlowDTO);
                        var purcostString=objectMapper.writeValueAsString(cashFlowDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000001_2,"+chuanyunProjectCashFlowDTO.getProjectcode());
                        var filter= Filter.instance(0,1,true,"And",macthers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunProjectCashFlowDO.SCHEMA_CODE,filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO =chuanYunManager.save(ChuanyunProjectCashFlowDO.SCHEMA_CODE,purcostString,true);
                        }else {
                            List<ChuanyunProjectCashFlowDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunProjectCashFlowDO chuanyunProjectCashFlowDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunProjectCashFlowDO.SCHEMA_CODE,chuanyunProjectCashFlowDO.getObjectId(),purcostString);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                            log.info("新增项目收付款成功");
                        }else{
                            log.warn("新增项目收付款失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 更新项目预决算统计
     */
    private void saveBudgetStatistics() {
        List<ChuanyunBudgetStatisticsDO> list = chuanyunBudgetStatisticsDao.getTotalByProject();
        list.forEach(chuanyunBudgetStatisticsDO -> {
            if (!StringUtils.isEmpty(chuanyunBudgetStatisticsDO.getProject())){
                var objectMapper = new ObjectMapper();
                BudgetDTO budgetDTO = new BudgetDTO(chuanyunBudgetStatisticsDO);
                budgetDTO.setDate(LocalDateTime.of(2020,4,1,0,0));
                try {
                    List<String> matchers = new ArrayList<>();
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    matchers.add("F0000001_2,"+chuanyunBudgetStatisticsDO.getProject());
                    matchers.add("F0000002_2,"+chuanyunBudgetStatisticsDO.getFeeType());
                    var filter= Filter.instance(0,1,true,"And",matchers);
                    var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunBudgetAmountDO.SCHEMA_CODE,filter);
                    var budgetString = objectMapper.writeValueAsString(budgetDTO);
                    if (chuanyunFindAllResponse.getReturnData() != null) {
                        List<ChuanyunBudgetAmountDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                        ChuanyunBudgetAmountDO chuanyunBudgetAmountDO = result.get(0);
                        chuanyunSaveDTO = chuanYunManager.update(ChuanyunBudgetAmountDO.SCHEMA_CODE,chuanyunBudgetAmountDO.getObjectId(),budgetString);
                    }else {
                        chuanyunSaveDTO = chuanYunManager.save(ChuanyunBudgetAmountDO.SCHEMA_CODE,budgetString,true);
                    }
                    if(chuanyunSaveDTO.getSuccessful()){
                            log.info("新增项目预决算统计成功");
                    }else{
                        log.warn("新增项目预决算统计失败");
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * 更新请假信息
     */
    private void saveAskForLeave() {
        var objectMapper = new ObjectMapper();
        //获取请假的信息
        List<ChuanyunAskForLeaveDO> askForLeaveList = chuanyunAskForLeaveDao.findAll();
        askForLeaveList.forEach(askForLeave -> {
            try {
                //根据钉钉号获取氚云用户Id
                ChuanyunUserDO chuanyunUserDO=chuanyunUserDao.findByDingTalkAccountContaining(askForLeave.getOriginatorUserid());
                if(chuanyunUserDO != null) {
                    askForLeave.setAskForLeaveUserId(chuanyunUserDO.getObjectId());
                }
                //根据dingTalkUserId获取chuanyunUser
                var purcostString = objectMapper.writeValueAsString(AskForLeaveDTO.fromChuanyunAskForLeaveDO(askForLeave));
                ChuanyunSaveDTO chuanyunSaveDTO;
                //根据项目号判断是否存在
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000001_2,"+askForLeave.getTitle());
                var filter= Filter.instance(0,1,true,"And",matchers);
                ChuanyunFindAllBizDTO<LinkedHashMap<String,String>> chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunAskForLeaveDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData() == null && (askForLeave.getIsCanncel() == 1)) {
                    return;
                }
                if(chuanyunFindAllResponse.getReturnData() == null && (askForLeave.getIsCanncel() == 0)){
                    //新增
                    chuanyunSaveDTO =chuanYunManager.save(ChuanyunAskForLeaveDO.SCHEMA_CODE,purcostString,true);
                }else if(chuanyunFindAllResponse.getReturnData() != null && (askForLeave.getIsCanncel() == 1)){
                    //删除撤销
                    chuanyunSaveDTO = chuanYunManager.delete(ChuanyunAskForLeaveDO.SCHEMA_CODE,chuanyunFindAllResponse.getReturnData().getBizObjectArray().get(0).get("ObjectId"));
                }else{
                    //更新
                    chuanyunSaveDTO = chuanYunManager.update(ChuanyunAskForLeaveDO.SCHEMA_CODE,chuanyunFindAllResponse.getReturnData().getBizObjectArray().get(0).get("ObjectId"),purcostString);
                }
                if(chuanyunSaveDTO.getSuccessful()){
//                        log.info("新增请假信息成功");
                }else{
                    log.warn("新增请假信息失败");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    private void saveSupplier() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取供应商信息
            PageDTO<ChuanyunSupplierDTO> pageInfo=clawFeign.findChuanyunSupplier(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunSupplierDTO> list = pageInfo.getContent();
            list.forEach(chuanyunSupplierDTO -> {
                SupplierDTO supplierDTO = new SupplierDTO(chuanyunSupplierDTO);
                try {
                    var supplierString = objectMapper.writeValueAsString(supplierDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    //根据编码判断供应商是否存在
                    List<String> macthers = new ArrayList<>();
                    macthers.add("F0000002_2,"+chuanyunSupplierDTO.getCode());
                    var filter= Filter.instance(0,1,true,"And",macthers);
                    var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSupplierDO.SCHEMA_CODE,filter);
                    if (chuanyunFindAllResponse.getReturnData() == null) {
                        chuanyunSaveDTO=chuanYunManager.save(ChuanyunSupplierDO.SCHEMA_CODE,supplierString,true);
                    }else {
                        List<ChuanyunSupplierDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                        chuanyunSaveDTO=chuanYunManager.update(ChuanyunSupplierDO.SCHEMA_CODE,result.get(0).getObjectId(),supplierString);
                    }
                    if(chuanyunSaveDTO.getSuccessful()){
                        log.info("新增供应商信息成功");
                    }else{
                        log.warn("新增供应商信息失败");
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }

            });
        }
    }

    /**
     * 更新项目物料成本
     */
    private void savePurCost() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取物料成本
            PageDTO<ChuanyunPurCostDTO> pageInfo = clawFeign.findChuanyunPurCost(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunPurCostDTO> list = pageInfo.getContent();
            list.forEach(chuanyunPurCostDTO -> {
                try {
                    if(chuanyunPurCostDTO != null) {
                        PurCostDTO purCostDTO = new PurCostDTO(chuanyunPurCostDTO);
                        var purcostString=objectMapper.writeValueAsString(purCostDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000001_2,"+chuanyunPurCostDTO.getProjectcode());
                        var filter= Filter.instance(0,1,true,"And",macthers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunPurCostDO.SCHEMA_CODE,filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO =chuanYunManager.save(ChuanyunPurCostDO.SCHEMA_CODE,purcostString,true);
                        }else {
                            List<ChuanyunPurCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunPurCostDO chuanyunPurCostDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunPurCostDO.SCHEMA_CODE,chuanyunPurCostDO.getObjectId(),purcostString);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
                            log.info("新增项目物料成本成功");
                        }else{
                            log.warn("新增项目物料成本失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 新增人力成本
     */
    @Scheduled(fixedDelay = 7200000L)
    private void saveLaborCost() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取物料成本
            PageDTO<ChuanyunLaborCostDTO> pageInfo = clawFeign.findChuanyunLaborCost(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunLaborCostDTO> list = pageInfo.getContent();
            list.forEach(chuanyunLaborCostDTO -> {
                try {
                    if(chuanyunLaborCostDTO != null) {
                        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
                        String newDate = dateTimeFormatter.format(chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod());
                        LocalDateTime dt = LocalDateTime.parse(newDate,dateTimeFormatter);
                        chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().setPeriod(dt);
                        LaborCostDTO laborCostDTO = new LaborCostDTO(chuanyunLaborCostDTO);
                        laborCostDTO.setDataType("U9");
                        var purcostString=objectMapper.writeValueAsString(laborCostDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000005_2,"+chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode());
                        matchers.add("F0000002_2,"+chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod());
                        var filter= Filter.instance(0,1,true,"And",matchers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunMonthLaborCostDO.SCHEMA_CODE,filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO =chuanYunManager.save(ChuanyunMonthLaborCostDO.SCHEMA_CODE,purcostString,true);
                        }else {
                            List<ChuanyunMonthLaborCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
//                            ChuanyunMonthLaborCostDO chuanyunMonthLaborCostDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunMonthLaborCostDO.SCHEMA_CODE,result.get(0).getObjectId(),purcostString);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                            log.info("新增项目人力成本成功");
                        }else{
                            log.warn("新增项目人力成本失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 获取供应商信息
     */
    private void getSupplier(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSupplierDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSupplierDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSupplierDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("供应商信息保存保存"+totalCount);
    }

    /**
     * 更新支出档案类型
     */
    public void getProjectType() {
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectTypeDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunProjectTypeDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunProjectTypeDao.saveAll(result);
            }catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info("支出档案类型"+totalCount);
        }
    }

    /**
     * 更新公司信息
     */
    private void getCompany(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCompanyDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCompanyDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCompanyDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("公司信息保存保存"+totalCount);
    }

    /**
     * 更新成本中心
     */
    private void getCost(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCostDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("成本中心保存保存"+totalCount);
    }

    /**
     * 获取专项分摊规则
     */
    private void getSpecialProcedure(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSpecialProcedureDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSpecialProcedureDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSpecialProcedureDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("专项分摊信息保存保存"+totalCount);
    }

    /**
     * 获取人员报销特殊项目
     */
    private void getReimbursementItems() {
        chuanyunReimbursementItemsDao.deleteAll();
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunReimbursementItemsDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunReimbursementItemsDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunReimbursementItemsDO->{
                    //同步外部项目名称
                    var chuanyunFindResponse =  chuanYunManager.find(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunReimbursementItemsDO.getAdjustProjectId());
                    ChuanyunGroupProjectDO tmpChuanyunGroupProjectDO = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>(){});
                    if (!chuanyunReimbursementItemsDO.getAdjustProjectName().equals(tmpChuanyunGroupProjectDO.getOutProjectName())) {
                        chuanyunReimbursementItemsDO.setAdjustProjectName(tmpChuanyunGroupProjectDO.getOutProjectName());
                        var reimbursementItemsDTO = new ReimbursementItemsDTO(chuanyunReimbursementItemsDO);
                        String groupProjectUpdateString= null;
                        try {
                            groupProjectUpdateString = objectMapper.writeValueAsString(reimbursementItemsDTO);
                            ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunReimbursementItemsDO.SCHEMA_CODE,chuanyunReimbursementItemsDO.getObjectId(),groupProjectUpdateString);
                            if(chuanyunSaveDTO.getSuccessful()) {
//                                log.info(chuanyunReimbursementItemsDO.getObjectId() + "同步外部项目名称成功" + groupProjectUpdateString);
                            }else{
                                log.warn(chuanyunReimbursementItemsDO.getObjectId()+"同步外部项目名称失败"+groupProjectUpdateString);
                            }
                        } catch (JsonProcessingException | BigSizeException e) {
                            e.printStackTrace();
                        }
                    }
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunReimbursementItemsDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("人员报销特殊项目保存保存"+totalCount);
    }

    /**
     * 获取人工成本特殊项目
     */
    private void getCostCollection(){
        chuanyunCostCollectionDao.deleteAll();
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCostCollectionDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCostCollectionDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunCostCollectionDO->{
                    //同步外部项目名称
                    var chuanyunFindResponse =  chuanYunManager.find(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunCostCollectionDO.getProjectId());
                    ChuanyunGroupProjectDO tmpChuanyunGroupProjectDO = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>(){});
                    //外部名称有修改就更新
                    if (!tmpChuanyunGroupProjectDO.getOutProjectName().equals(chuanyunCostCollectionDO.getOutProjectName())) {
                        chuanyunCostCollectionDO.setOutProjectName(tmpChuanyunGroupProjectDO.getOutProjectName());
                        var costCollectionDTO = new CostCollectionDTO(chuanyunCostCollectionDO);
                        String groupProjectUpdateString= null;
                        try {
                            groupProjectUpdateString = objectMapper.writeValueAsString(costCollectionDTO);
                            ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunCostCollectionDO.SCHEMA_CODE,chuanyunCostCollectionDO.getObjectId(),groupProjectUpdateString);
                            if(chuanyunSaveDTO.getSuccessful()) {
                                log.info(chuanyunCostCollectionDO.getObjectId() + "同步外部项目名称成功" + groupProjectUpdateString);
                            }else{
                                log.warn(chuanyunCostCollectionDO.getObjectId()+"同步外部项目名称失败"+groupProjectUpdateString);
                            }
                        } catch (JsonProcessingException | BigSizeException e) {
                            e.printStackTrace();
                        }
                    }
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCostCollectionDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("人工成本特殊项目保存保存"+totalCount);
    }

    /**
     * 获取人员-归属公司信息
     */
    private void getUserCompany(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunUserCompanyDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunUserCompanyDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunUserCompanyDO -> {
                    try {
                        if(chuanyunUserCompanyDO.getUser()!=null){
                            chuanyunUserCompanyDO.setUserId(chuanyunUserCompanyDO.getUser().getObjectId());
                        }
                        if (!StringUtils.isEmpty(chuanyunUserCompanyDO.getDepartment())) {
                            chuanyunUserCompanyDO.setDepartmentId(chuanyunUserCompanyDO.getDepartmentObject().getObjectId());
                        }
                    }catch (Exception exception){
                        log.warn(chuanyunUserCompanyDO+"");
                        log.warn(exception.getMessage());
                    }
                    try{
                        ChuanYunConstant.USER_BUSINESS_GROUP.put(chuanyunUserCompanyDO.getUserId(),chuanyunCompanyDao.findByObjectId(chuanyunUserCompanyDO.getCompanyId()).getBusinessGroup());
                    }catch (Exception e){
                        log.warn("更新用户BG信息时发生异常"+e.getMessage());
                    }
                    ChuanYunConstant.USER_DEPARTMENT.put(chuanyunUserCompanyDO.getUserId(),chuanyunUserCompanyDO.getDepartmentObject().getObjectId());
                });
                chuanyunUserCompanyDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("人员归属公司信息保存保存"+totalCount);
    }

    /**
     * 获取报销科目
     */
    private void getReimbursementSubject(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunReimbursementSubjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunReimbursementSubjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunReimbursementSubjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("报销科目信息保存保存"+totalCount);
    }

    /**
     * 同步集团总表（手动更新调用）
     */
    private void getGroupProjectForSynchronization(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {
                    var tmpGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunGroupProjectDO.getObjectId());
                    tmpGroupProjectDO.ifPresent(projectDO -> {
                        chuanyunGroupProjectDO.setIsCreate(projectDO.getIsCreate());
                        chuanyunGroupProjectDao.delete(tmpGroupProjectDO.get());
                    });
                });
                chuanyunGroupProjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("集团项目信息保存保存"+totalCount+"项目映射更新完成");
    }

    @Data
    class GroupProjectDetailDTO {

        @JsonProperty("D001789Fvspc68z01idftf5nqt66pkuc0")
        private List<ChuanyunGroupProjectDetailDO> detailList;

        public GroupProjectDetailDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if (null!=chuanyunGroupProjectDO) {
                if (null!=chuanyunGroupProjectDO.getDetailList()) {
                    this.detailList = chuanyunGroupProjectDO.getDetailList();
                }
            }
        }

    }

    @Data
    class CostCollectionDTO {
        /**
         * 外部项目名称
         */
        @JsonProperty("F0000008")
        public String outProjectName;

        public CostCollectionDTO(ChuanyunCostCollectionDO chuanyunCostCollectionDO) {
            if (null!=chuanyunCostCollectionDO) {
                if (null!=chuanyunCostCollectionDO.getOutProjectName()) {
                    this.outProjectName = chuanyunCostCollectionDO.getOutProjectName();
                }
            }
        }
    }

    @Data
    class ReimbursementItemsDTO {
        /**
         * 调整项目名称
         */
        @JsonProperty("F0000007")
        private String adjustProjectName;

        public ReimbursementItemsDTO(ChuanyunReimbursementItemsDO chuanyunReimbursementItemsDO) {
            if (null!=chuanyunReimbursementItemsDO) {
                if (null!=chuanyunReimbursementItemsDO.getAdjustProjectName()) {
                    this.adjustProjectName = chuanyunReimbursementItemsDO.getAdjustProjectName();
                }
            }
        }
    }

    @Data
    class BudgetDTO{

        /**
         * 所属项目
         */
        @JsonProperty("F0000001")
        private String project;

        /**
         * 费用类型
         */
        @JsonProperty("F0000002")
        private String feeType;

        /**
         * 预算金额
         */
        @JsonProperty("F0000003")
        private BigDecimal budgetAmount;

        /**
         * 默认日期（4月份）
         */
        @JsonProperty("F0000006")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime date;

        /**
         * 决算金额
         */
        @JsonProperty("F0000004")
        private BigDecimal finalAmount;

        /**
         * 预算绝算差异
         */
        @JsonProperty("F0000005")
        private BigDecimal differenceAmount;

        public BudgetDTO(ChuanyunBudgetStatisticsDO chuanyunBudgetStatisticsDO) {
            if (null!=chuanyunBudgetStatisticsDO){
                if (null!=chuanyunBudgetStatisticsDO.getBudgetAmount()){
                    this.budgetAmount=chuanyunBudgetStatisticsDO.getBudgetAmount();
                }
                if (null!=chuanyunBudgetStatisticsDO.getFinalAmount()){
                    this.finalAmount=chuanyunBudgetStatisticsDO.getFinalAmount();
                }
                if (null!=chuanyunBudgetStatisticsDO.getFeeType()){
                    this.feeType=chuanyunBudgetStatisticsDO.getFeeType();
                }
                if (null!=chuanyunBudgetStatisticsDO.getProject()){
                    this.project=chuanyunBudgetStatisticsDO.getProject();
                }
                if (null!=chuanyunBudgetStatisticsDO.getDifferenceAmount()){
                    this.differenceAmount=chuanyunBudgetStatisticsDO.getDifferenceAmount();
                }
            }
        }
    }

    @Data
    class CashFlowDTO {
        @JsonProperty("F0000001")
        private String projectcode;

        /**
         * 收款金额
         */
        @JsonProperty("F0000002")
        private BigDecimal income;

        /**
         * 付款金额
         */
        @JsonProperty("F0000003")
        private BigDecimal outcome;

        public CashFlowDTO(ChuanyunProjectCashFlowDTO chuanyunProjectCashFlowDTO) {
            if (null!=chuanyunProjectCashFlowDTO) {
                if (null!=chuanyunProjectCashFlowDTO.getProjectcode()) {
                    this.projectcode=chuanyunProjectCashFlowDTO.getProjectcode();
                }
                if (null!=chuanyunProjectCashFlowDTO.getOutcome()) {
                    this.outcome=chuanyunProjectCashFlowDTO.getOutcome();
                }
                if (null!=chuanyunProjectCashFlowDTO.getIncome()) {
                    this.income=chuanyunProjectCashFlowDTO.getIncome();
                }
            }
        }
    }

    @Data
    class SupplierDTO{
        /**
         * 供应商名称
         */
        @JsonProperty("F0000001")
        private String supplierName;

        /**
         * U9代码
         */
        @JsonProperty("F0000002")
        private String supplierCode;

        public SupplierDTO(ChuanyunSupplierDTO chuanyunSupplierDTO) {
            if (null!=chuanyunSupplierDTO){
                if (null!=chuanyunSupplierDTO.getCode()){
                    this.supplierCode=chuanyunSupplierDTO.getCode();
                }
                if (null!=chuanyunSupplierDTO.getName()){
                    this.supplierName=chuanyunSupplierDTO.getName();
                }
            }
        }
    }

    @Data
    class PurCostDTO {
        @JsonProperty("F0000001")
        private String projectcode;

        /**
         * 采购成本
         */
        @JsonProperty("F0000002")
        private BigDecimal purcost;

        /**
         * 领料成本
         */
        @JsonProperty("F0000003")
        private BigDecimal issuedcost;

        public PurCostDTO(ChuanyunPurCostDTO chuanyunPurCostDTO) {
            if (null!=chuanyunPurCostDTO) {
                if (null != chuanyunPurCostDTO.getProjectcode()) {
                    this.projectcode = chuanyunPurCostDTO.getProjectcode();
                }
                if (null != chuanyunPurCostDTO.getPurcost()) {
                    this.purcost = chuanyunPurCostDTO.getPurcost();
                }
                if (null != chuanyunPurCostDTO.getIssuedcost()) {
                    this.issuedcost = chuanyunPurCostDTO.getIssuedcost();
                }
            }
        }
    }

    @Data
    class LaborCostDTO {

        @JsonProperty("F0000005")
        private String projectCode;

        @JsonProperty("F0000003")
        private BigDecimal amount;

        @JsonProperty("F0000002")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime logDate;

        @JsonProperty("F0000004")
        private String dataType;

        public LaborCostDTO(ChuanyunLaborCostDTO chuanyunMonthLaborCostDTO) {
            if (null!=chuanyunMonthLaborCostDTO){
                if (null!=chuanyunMonthLaborCostDTO.getCostamount()){
                    this.amount=chuanyunMonthLaborCostDTO.getCostamount();
                }
                if (null!=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode()){
                    this.projectCode=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode();
                }
                if (null!=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod()){
                    this.logDate=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod();
                }
            }
        }
    }

    /**
     * 用于刷新项目经理权限的对象
     */
    @Data
    class GroupProjectUpdateDTO{
        /**
         * 项目总监
         * U9项目责任人
         */
        @JsonProperty(value = "F0000006")
        private String projectOwner;

        /**
         * IBG项目经理
         */
        @JsonProperty("F0000012")
        private String ibgProjectManager;

        /**
         * EBG项目经理
         */
        @JsonProperty("F0000011")
        private String ebgProjectManager;

        /**
         * 外部项目名称
         */
        @JsonProperty(value="F0000038")
        private String outProjectName;


        public GroupProjectUpdateDTO() {
        }

        public GroupProjectUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
                if(null!=chuanyunGroupProjectDO.getProjectOwnerObject()){
                    this.projectOwner=chuanyunGroupProjectDO.getProjectOwnerObject().getObjectId();
                }
                if(null!=chuanyunGroupProjectDO.getIbgProjectManagerObject()){
                    this.ibgProjectManager=chuanyunGroupProjectDO.getIbgProjectManagerObject().getObjectId();
                }
                if(null!=chuanyunGroupProjectDO.getEbgProjectManagerObject()){
                    this.ebgProjectManager=chuanyunGroupProjectDO.getEbgProjectManagerObject().getObjectId();
                }
                if (null!=chuanyunGroupProjectDO.getOutProjectName()) {
                    this.outProjectName = chuanyunGroupProjectDO.getOutProjectName();
                }
            }
        }
    }

    @Data
    static class GroupProjectMemberListUpdateDTO{
        /**
         * 项目组成员（前端传数据用）
         */
        @JsonProperty("F0000034")
        private String[] projectMemberList;

        public void add(){
            String[] newProjectMemberList=new String[projectMemberList.length+1];
            System.arraycopy(projectMemberList,0,newProjectMemberList,0,projectMemberList.length);
            newProjectMemberList[projectMemberList.length]="8bbb21c4-1140-4bfa-8713-6ee50ea34df0";
            projectMemberList=newProjectMemberList;
        }

        public void delete(){
            String[] newProjectMemberList=new String[projectMemberList.length-1];
            System.arraycopy(projectMemberList,0,newProjectMemberList,0,projectMemberList.length-1);
            projectMemberList=newProjectMemberList;
        }

        public GroupProjectMemberListUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
                if(null!=chuanyunGroupProjectDO.getProjectMemberList()){
                    List<String> memberList = new ArrayList<>();
                    chuanyunGroupProjectDO.getProjectMemberObject().forEach(chuanyunObject -> memberList.add(chuanyunObject.getObjectId()));
                    this.projectMemberList = memberList.toArray(new String[0]);
                }
            }
        }
    }

    @Data
    static class AskForLeaveDTO{

        /**
         * 标题
         */
        @JsonProperty("F0000001")
        private String title;

        /**
         * 发起人姓名
         */
        @JsonProperty("F0000007")
        private String originatorUserName;

        /**
         * 发起部门
         */
        @JsonProperty("F0000002")
        private String originatorDeptName;

        /**
         * 请假事由
         */
        @JsonProperty("F0000005")
        private String leaveType;

        /**
         * 劳动合同所属公司
         */
        @JsonProperty("F0000004")
        private String contractOwnerLtd;

        @JsonProperty("F0000006")
        private String result;

        /**
         * 请假开始时间
         */
        @JsonProperty("F0000016")
        private String startTime;

        /**
         * 请假时长
         */
        @JsonProperty("F0000017")
        private BigDecimal totalDay;

        /**
         * 请假人Id
         */
        @JsonProperty("F0000018")
        private String askForLeaveUserId;

        public static AskForLeaveDTO fromChuanyunAskForLeaveDO(ChuanyunAskForLeaveDO chuanyunAskForLeaveDO) {
            AskForLeaveDTO askForLeaveDTO=new AskForLeaveDTO();
            BeanUtils.copyProperties(chuanyunAskForLeaveDO,askForLeaveDTO);
            return askForLeaveDTO;
        }
    }
}
